import os
import random


AUTHOR = 'Denis Druzhinin'
SAVE_FOLDER = os.path.normpath(
    'c:/Users/Denis/Documents/u-he/Hive.data/Wavetables/drzhnn - Lab/'
)
NUM_FILES = 100


def rescale(lst):
    """Takes a list of numeric values and rescale them so that their
    total is close to 1.0. Returns a rescaled list.
    """
    r = round(1 / sum(lst), 3)
    new_list = [round(i * r, 3) for i in lst]
    return new_list


def generate_envelope():
    """Generates lists of random values, rescales, applies shuffle
    and uses them to build envelope points. Returns a complete
    Envelope line as a string.
    """
    tt = [random.random() for i in range(5)]
    t = rescale(tt)
    ll = [random.random() for i in range(6)]
    l = rescale(ll)
    random.shuffle(l)
    string = ('Envelope '
              f'L0={l[0]} '
              f'T1={t[0]} '
              f'L1={l[1]} '
              f'T2={t[1]} '
              f'L2={l[2]} '
              f'T3={t[2]} '
              f'L3={l[3]} '
              f'T4={t[3]} '
              f'L4={l[4]} '
              f'T5={t[4]} '
              f'L5={l[5]} '
              )
    return string


def generate_filtered_noise(type, cutoff_lowest, cutoff_highest, resonance, volume=None):
    """Randomly selects a filter type. Generates random values for seed, cutoff,
    resonance, volume and envelope curvature.
    Returns two lines (Seed..., Wave...) as a formatted string.
    """
    ft = ['lowpass', 'bandpass', 'highpass']
    if type == 4:
        t = random.choice(ft)
    else:
        t = ft[type]
    c = round(random.uniform(cutoff_lowest, cutoff_highest), 3)
    if t is 'bandpass':
        r = round(random.uniform(0.5, 0.9), 3)
    else:
        r = round(random.uniform(0.0, resonance), 3)
    p = round(random.uniform(0.1, 2.7), 3)
    et = ['', f'*env(phase)', f'*env(phase)^{p}']
    e = random.choice(et)
    if not volume == None:
        vol = round(random.uniform(0.02, volume), 3)
        v = f' * {vol}{e}'
    else:
        v = ''
    seed = random.randint(1, 99999999999)
    string = (f'Seed = {seed}\n'
              f'Wave "{t}(1-2*rand, {c}{e}, {r}){v}"')
    return string


def generate_uhm(fullpath):

    with open(fullpath, 'w+') as f:
        data = (f'// by {AUTHOR}\n\n'
                'NumFrames = 1\n'
                f'{generate_envelope()}\n'
                f'{generate_filtered_noise(0, 0.10, 0.15, 0.2)}\n'
                f'{generate_envelope()}\n'
                f'{generate_filtered_noise(1, 0.35, 0.55, 0.5, 0.5)} blend=add\n'
                f'{generate_envelope()}\n'
                f'{generate_filtered_noise(1, 0.55 ,0.65, 0.4, 0.03)} blend=add\n'
                'Spectrum lowest=0 highest=0 "0"\n'
                'Normalize base=each\n'
                )
        f.write(data)


def main(num_files):
    for i in range(num_files):
        filename = f"Wave {i+1:03}.uhm"
        fullpath = os.path.join(SAVE_FOLDER, filename)
        generate_uhm(fullpath)


if __name__ == '__main__':
    if not os.path.exists(SAVE_FOLDER):
        os.mkdir(SAVE_FOLDER)

    main(NUM_FILES)
